# Stateless function of generating random large prime given the bit-length, return as utils.data_structure.binary.Binary.
# Stateless function of generating private key from two large prime numbers, return as utils.data_structure.binary.Binary.
# Stateless function of generating public key from two large prime numbers, return as utils.data_structure.binary.Binary.
# Stateless function of digital signature or encrypt from private key or public key on the message, input as utils.data_structure.binary.Binary, return as utils.data_structure.binary.Binary.
# Stateless boolean function to verify digital signed message (utils.data_structure.binary.Binary) with public key or private key.