# Stateless function of generating private key from arbitrary passphrase, return as utils.data_structure.binary.Binary.
# Stateless function of generating public key from private key, return as utils.data_structure.binary.Binary.
# Stateless function of generating account address from public key, return as utils.data_structure.binary.Binary.
# Stateless function of digital signature from private key and message (utils.data_structure.binary.Binary), return as utils.data_structure.binary.Binary.
# Stateless boolean function to verify digital signed message (utils.data_structure.binary.Binary) with public key (utils.data_structure.binary.Binary).